#!/bin/bash

# This script runs instant3Dhub on a single node setup with CentOS 8.  
# It uses microk8s to deploy and run the system.
# Prerequisites:
#   1- Snapd is installed and running
#   2- SELinux is disabled
#   3- The system has the correct Linux kernel sources from the CentOS repositories
#   4- If you want to use GPU support, then make sure that nouveau dirver is disabled
#   5- If your system uses firewalld then make sure that masquerade is enabled on the host.
#   6- You have a running instant3Dhub License Server which can be accessed from the host. 
# If you are not sure if or how to make your system meets these requirments, then please check and run the prerequisite.sh script which will reconfigure your system to meet the requirments 1-5.
# For installing instant3Dhub License Server please check the documenation in LICENSE_SERVER.rst. 
# Running the script:
# The script expects these env variables to be set in the setup file:
#   RELEASE_NAME: Is the name of the release to be used when deploying instant3Dhub. This can be any name of your choise. Example: instant3dhub-test. Default is: i3dhub-singlenode.
#   ENTRYPOINT: The external exposure hosts or proxies. Example: http://host-name.your-domain.com:30000. Default is: http://your-hostname:30042.
#   REGISTRY: Is the containers registry where instant3Dhub images can be fetched. Example: hub.docker.com. Default is: images.threedy.io.
#   LICENSE_SERVER: The address of the license server. Example: http://license.threedy.io:8200. Default is empty. 
#   ENABLE_GPU: Set to true if your system has GPU and you want to enable them to be used by instant3Dhub visualization services. Default is false. 
# Please adjust these variables based on your setup.

###############################################################################################
# Read env variables
###############################################################################################
cd ${0%/*}

source ./setup

# Set default values
RELEASE_NAME="${RELEASE_NAME:-i3dhub-singlenode}"
ENTRYPOINT="${ENTRYPOINT:-$(hostname):30042}"
REGISTRY="${REGISTRY:-images.threedy.io}"
LICENSE_SERVER="${LICENSE_SERVER:-http://license.threedy.io:8200}"
ENABLE_GPU="${ENABLE_GPU:-false}"
ENTRYPOINT_PORT="${ENTRYPOINT_PORT:-30042}"

#################################################################################################
# Install microk8s
################################################################################################# 

echo Installing and configuring microk8s

# Install microk8s
snap install microk8s --classic --channel=1.22/stable

# Add the user to the microk8s group
usermod -aG microk8s $USER
chown -f -R $USER ~/.kube

# Enable storage
export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:/lib64"
microk8s enable storage

# Enable dns and make sure it uses the local dns
microk8s enable dns:/etc/resolv.conf

# Enable helm3 
microk8s enable helm3

if [ "$ENABLE_GPU" = true ] ; then
    # Enable GPU 
    microk8s enable gpu

    # Wait till kubernetes resources for enabling GPU support are created 
    while ! microk8s kubectl get ds -n gpu-operator-resources | grep -q 'nvidia-device-plugin-daemonset'
    do 
        echo -ne Waiting for kubernetes GPU-enabling-resources to be available \\r
        sleep 20
    done

    # Make sure nvidia driver daemonset uses the latest nvidia image
    microk8s kubectl set image ds/nvidia-driver-daemonset nvidia-driver-ctr=nvcr.io/nvidia/driver:470.57.02-centos8  -n gpu-operator-resources

    # This is needed for GPU plugin to work. nvidia device plugin daemonset expects the file ldconfig to have the extension real. 
    until [ -f /run/nvidia/driver/sbin/ldconfig ]
    do
        echo -ne Waiting for nvidia driver to be installed \\r
        sleep 30
    done
    echo NVIDIA driver is installed

    # Link ldconfig.real to ldconfig. 
    ln  /run/nvidia/driver/sbin/ldconfig   /run/nvidia/driver/sbin/ldconfig.real

fi

# Show the status of microk8s 
microk8s status

##############################################################################
# Deploy instant3Dhub
##############################################################################

echo Deploying instant3Dhub ...

# Create namespace
microk8s kubectl create ns ${RELEASE_NAME}



# Deploy instant3Dhub. For more information about the avialable config options, please check the values.yaml file in helm folder.
extra_params=()
if [ "$ENABLE_GPU" = true ] ; then
    extra_params+=(--set numGPUVisPods=1)
fi


microk8s helm3 install -n ${RELEASE_NAME} ${RELEASE_NAME} ../../helm \
    --wait \
    --timeout 150s \
	--set entrypoints={${ENTRYPOINT}} \
	--set registry=${REGISTRY} \
	--set licenseServer=${LICENSE_SERVER} \
	--set storage.class=microk8s-hostpath \
    --set entryNodePort=${ENTRYPOINT_PORT} \
    "${extra_params[@]}"

# Allow access to the entry port through the firewall
sudo firewall-cmd  --permanent --add-port=${ENTRYPOINT_PORT}/tcp
sudo firewall-cmd  --reload

echo "instant3Dhub is installed and can be accessed at: ${ENTRYPOINT}"
